import dash
from webviz_config.common_cache import cache
from pytest_dash.wait_for import (
    wait_for_element_by_css_selector as css_select
)
from webviz_config.containers import _example_container


def test_example_container(dash_threaded):

    app = dash.Dash(__name__)
    app.css.config.serve_locally = True
    app.scripts.config.serve_locally = True
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)
    driver = dash_threaded.driver
    title = 'Example'
    page = _example_container.ExampleContainer(app, title)
    app.layout = page.layout
    dash_threaded(app)
    btn = css_select(driver, f'#{page.button_id}')
    assert 'Submit' == btn.text
    text = css_select(driver, f'#{page.div_id}')
    assert 'Button has been pressed 0 times.' == text.text
    btn.click()
    assert 'Button has been pressed 1 times.' == text.text



import abc


class WebvizContainer(abc.ABC):
    '''All webviz containers need to subclass this abstract base class,
    e.g.

    ```python
    class MyContainer(WebvizContainer):

        def __init__(self):
            ...

        def layout(self):
            ...
    ```
    '''

    # List of container specific assets which should be copied
    # over to the ./assets folder in the generated webviz app.
    # This is typically custom JavaScript and/or CSS files.
    # All paths in the returned ASSETS list should be absolute.
    ASSETS = []

    @abc.abstractmethod
    def layout(self):
        '''This is the only required function of a Webviz Container.
        It returns a Dash layout which by webviz-config is added to
        the main Webviz application.
        '''
        pass



from uuid import uuid4
from pathlib import Path
import dash_core_components as dcc
from . import WebvizContainer
from ..webviz_store import webvizstore


class SyntaxHighlighter(WebvizContainer):
    '''### Syntax highlighter

This container adds support for syntax highlighting of code. Language is
automatically detected.

* `filename`: Path to a file containing the code to highlight.
* `show_line_numbers`: If `True`, show line numbers. Default is `False`.
* `start_number`: This only has effect if `show_line_numbers` is `True`. If
                  given, line numbers will start at this value. Default is `1`.
* `dark_theme`: If `True`, the code is shown with a dark theme. Default is
                `False` giving a light theme.
'''

    def __init__(self, filename: Path, show_line_numbers: bool = False,
                 start_number: int = 1, dark_theme: bool = False):

        self.filename = filename
        self.show_line_numbers = show_line_numbers
        self.start_number = start_number
        self.dark_theme = dark_theme

        self.syntax_highlighter_id = 'syntax-highlighter-{}'.format(uuid4())

        with get_path(self.filename).open() as fh:
            self.code = fh.read()

    def add_webvizstore(self):
        return [(get_path, [{'path': self.filename}])]

    @property
    def layout(self):
        return dcc.SyntaxHighlighter(
                 id=self.syntax_highlighter_id,
                 children=self.code,
                 showLineNumbers=self.show_line_numbers,
                 startingLineNumber=self.start_number,
                 theme='dark' if self.dark_theme else 'light'
                        )


@webvizstore
def get_path(path) -> Path:
    return path
